package me.tofaa.entitylib.codegen;

import com.squareup.javapoet.ClassName;
import com.squareup.javapoet.JavaFile;
import com.squareup.javapoet.TypeSpec;

import javax.lang.model.element.Modifier;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static me.tofaa.entitylib.codegen.MetadataClass.createMetadataClass;

public final class Main {

    private Main() {}
    static final File OUT = new File("./code-gen/autogenerated/");
    static {
        if (!OUT.exists()) {
            OUT.mkdirs();
        }
        if (!OUT.isDirectory()) {
            OUT.delete();
            OUT.mkdirs();
        }
    }

    public static void main(String[] args) throws IOException {
        createMetadataClass();
        List<MetaClass> metaClasses = new ArrayList<>();

        // test
        MetaMethod method = new MetaMethod("username", String.class, "\"\"", "EntityDataType.STRING", List.of(new VersionCheck(47, 47, (byte) 0)));
        MetaClass obj = new MetaClass("me.tofaa.entitylib.meta:EntityMeta", null, List.of(method));

        writeClass(obj);
    }

    private static void writeClass(MetaClass metaClass) {
        TypeSpec typeSpec = metaClass.create();
        JavaFile javaFile = JavaFile.builder("me.tofaa.entitylib.meta", typeSpec).build();
        try {
            javaFile.writeToPath(OUT.toPath());
        }
        catch (IOException e)  {
            throw new RuntimeException(e);
        }
    }


    // Include comment in generated code
    public static TypeSpec.Builder builder(String packageName, String className) {
        return TypeSpec.classBuilder(ClassName.get(packageName, className))
                .addModifiers(Modifier.PUBLIC)
                .addJavadoc("AUTOGENERATED CODE BY ENTITYLIB CODEGEN\n")
                .addJavadoc("DO NOT EDIT\n");
    }


}
